package graph;

import java.awt.*;
import java.applet.*;
import java.net.URL;
import java.util.*;
import java.io.*;

/*
**************************************************************************
**
**    Class  DataSetPrinter
**
**************************************************************************
** Copyright (c) 1998
**      The Regents of the University of California.  All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions
** are met:
** 1. Redistributions of source code must retain the above copyright
**    notice, this list of conditions and the following disclaimer.
** 2. Redistributions in binary form must reproduce the above copyright
**    notice, this list of conditions and the following disclaimer in the
**    documentation and/or other materials provided with the distribution.
** 3. All advertising materials mentioning features or use of this software
**    must display the following acknowledgement:
**      This product includes software developed by the University of
**      California, Berkeley and its contributors.
** 4. Neither the name of the University nor the names of its contributors
**    may be used to endorse or promote products derived from this software
**    without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
** ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
** ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
** FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
** DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
** OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
** HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
** LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
** OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
** SUCH DAMAGE.
**
** Written by Mark Stemm (stemm@cs.berkeley.edu)
**************************************************************************
**
**    This class takes as input a Data Set and can generate postscript
**    commands that show the data set.
**
*************************************************************************/

/**
 * This class takes as input a DataSet class and generates postscript
 * using a "print to file" method. It depends on postscript commands that are
 * defined earlier in the postscript file (by the GraphPrinter object's
 * drawToFile method).
 * @version $Revision: 1.2 $, $Date: 1998/04/13 22:20:42 $.
 * @author Mark Stemm (stemm@cs.berkeley.edu)
 */

public class DataSetPrinter {
 /*
*******************
** Public Variables
******************/
  /**
   * The DataSet object to generate the postscript for.
   */
  DataSet d; 
  
    /*
*******************
** Constructors
******************/

  public DataSetPrinter(DataSet d0){
    d=d0;
  }

   /*
*******************
** Public Methods
******************/
  
  /**
   * Output postcript commands to the stream file that draw the
   * axis. 
   */
  public void drawToFile(PrintStream file)
  {
    int i;
    
    double[] p;
    MarkersPrinter m = new MarkersPrinter((Markers) d.g2d.getMarkers());

    // Draw a line connecting all the data points
    p = d.getPoint(0);
    file.println(p[0]+" xunits "+p[1]+" yunits "+" moveto");

    for(i=1; i<d.dataPoints(); i++){
      p = d.getPoint(i);
      file.println(p[0]+" xunits "+p[1]+" yunits "+" lineto");
    }

    file.println("stroke");

    // Now draw symbols at the data points
    if (m == null) return;
    
    for(i=1; i<d.dataPoints(); i++){
      p = d.getPoint(i);
      m.drawToFile(file, d.marker, d.markerscale, (int) p[0], (int) p[1]);
    }
  }
}
  
