package graph;

import java.awt.*;
import java.applet.*;
import java.net.URL;
import java.util.*;
import java.io.*;

/*
**************************************************************************
**
**    Class GraphPrinter
**
**************************************************************************
** Copyright (c) 1998
**      The Regents of the University of California.  All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions
** are met:
** 1. Redistributions of source code must retain the above copyright
**    notice, this list of conditions and the following disclaimer.
** 2. Redistributions in binary form must reproduce the above copyright
**    notice, this list of conditions and the following disclaimer in the
**    documentation and/or other materials provided with the distribution.
** 3. All advertising materials mentioning features or use of this software
**    must display the following acknowledgement:
**      This product includes software developed by the University of
**      California, Berkeley and its contributors.
** 4. Neither the name of the University nor the names of its contributors
**    may be used to endorse or promote products derived from this software
**    without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
** ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
** ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
** FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
** DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
** OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
** HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
** LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
** OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
** SUCH DAMAGE.
**
** Written by Mark Stemm (stemm@cs.berkeley.edu)
**************************************************************************
**
**    This class allows one to print a graph to a postscript file.
**
*************************************************************************/

/**
 *    This class takes as input a Graph object, and generates a postscript
 *    representation of the graph.
 *
 * @version $Revision: 1.1 $, $Date: 1998/05/13 02:13:57 $.
 * @author Mark Stemm (stemm@cs.berkeley.edu)
 */

public class GraphPrinter {

      /*
*******************
** Public Variables
******************/
  /**
   * The Graph object to generate the postscript for.
   */
  Graph2D g; 
  
    /*
*******************
** Constructors
******************/

  public GraphPrinter(Graph2D g0){
    g=g0;
  }

  /*
*******************
** Public Methods
******************/
  
  /**
   * Print the current graph to a postscript file specified by
   * <i>file</i>.  Defines functions that map coordinate space of
   * graph into coordinate space of postscript paper, and then calls
   * Axis and DataSet graphing functions.  BUG: This function does not
   * work when axes have different units (i.e. xaxis on left is
   * inches, axis on right is cm, etc)
   */
  public void drawToFile(PrintStream file)
  {
    Enumeration axisEnum;
    Enumeration dataSetEnum;
    double xmax=java.lang.Double.NEGATIVE_INFINITY;
    double ymax=java.lang.Double.NEGATIVE_INFINITY;
    double xmin=java.lang.Double.POSITIVE_INFINITY;
    double ymin=java.lang.Double.POSITIVE_INFINITY;

    double xrange, yrange, xscale, yscale;
    
    // Scan over axes attached to graph to find bounding box
    axisEnum = g.axis.elements();

    while(axisEnum.hasMoreElements()){
       Axis a = (Axis) axisEnum.nextElement();
       switch (a.position){
       case Axis.LEFT: case Axis.RIGHT:
	 if (a.maximum>ymax) ymax=a.maximum;
	 if (a.minimum<ymin) ymin=a.minimum;
	 break;
       case Axis.TOP: case Axis.BOTTOM:
	 if (a.maximum>xmax) xmax=a.maximum;
	 if (a.minimum<xmin) xmin=a.minimum;
	 break;
       }
    }

    xrange=xmax-xmin;
    yrange=ymax-ymin;
    xscale=72*9/xrange;
    yscale=72*6.5/yrange;
    
    file.println("/xunits {"+xmin+" sub "+xscale+" mul} def");
    file.println("/yunits {"+ymin+" sub "+yscale+" mul} def");

    file.println("90 rotate");
    file.println("90 -560 translate");
    file.println("/Times-Roman findfont");
    file.println("18 scalefont");
    file.println("setfont");

    // Draw the Data Sets attached to the graph
    dataSetEnum = g.dataset.elements();

    while(dataSetEnum.hasMoreElements()){
      DataSetPrinter d = new DataSetPrinter((DataSet) dataSetEnum.nextElement());
      d.drawToFile(file);
    }

    // Draw the Axes attached to the graph
    axisEnum = g.axis.elements();

    while(axisEnum.hasMoreElements()){

      Axis a = (Axis) axisEnum.nextElement();
      AxisPrinter ap = new AxisPrinter(a);

      switch (a.position){
       case Axis.LEFT: case Axis.RIGHT:
	 ap.drawToFile(file, xmin, xmax);
	 break;
       case Axis.TOP: case Axis.BOTTOM:
	 ap.drawToFile(file, ymin, ymax);
	 break;
       }
    }

    file.println("showpage");
  }
}

  
