/*
 * Copyright (c) 1996-1998 Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the SPAND Research
 *      Group at the University of California Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
/* $Header: /home/stemm/cvs/JavaSpand/spand/ErrorPayload.java,v 1.4 1998/06/27 01:41:18 stemm Exp $ */
package spand;

/** An ErrorPayload is used to return error messages. It just contains
 *  an error string.
 * @version $Revision: 1.4 $.
 * @author SPAND Group, UC Berkeley.
 */

public class ErrorPayload implements SendableSpandObject {

  /** The actual error message */
  public String errorMsg;

  /** Default constuctor */
  public ErrorPayload(){
    errorMsg=null;
  }

  /** Constructor from explicit message */
  public ErrorPayload(String errorMsg0){
    errorMsg=errorMsg0;
  }

  /** Unpackage this object from a sequence of bytes. The number of
      bytes read is returned. */
  public int Unmarshal(byte[] wireData, int startIndex){

    int pos=startIndex;
    
    errorMsg = ArrayConv.ByteArrayToString(wireData, pos);

    pos+=(errorMsg.length()+4);
    while ((pos%4)!=0){
      pos++;
    }
    return (pos-startIndex);
  }

  /** Package this object into a sequence of bytes. The number of
      bytes written is returned. If wireData is NULL, then no bytes
      are actually written, but the size of the marshalled object is
      returned. */
  public int Marshal(byte[] wireData, int startIndex){
    if (wireData==null){
      int unalign_size=errorMsg.length()+4;
      while((unalign_size%4)!=0){
        unalign_size++;
      }
      return (unalign_size);
    }

    int pos=startIndex;
    
    ArrayConv.StringToByteArray(wireData, pos, errorMsg);
    pos+=(errorMsg.length()+4);
    while((pos%4)!=0){
      pos++;
    }
    
    return (pos-startIndex);
  }

  /** Return a String representation of this object. */
  public String toString(){
    return errorMsg;
  }
  
}

