/*
 * Copyright (c) 1996-1998 Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the SPAND Research
 *      Group at the University of California Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
/* $Header: /home/stemm/cvs/JavaSpand/spand/GetSumm.java,v 1.6 1998/06/24 21:41:43 stemm Exp $ */
package spand;
import java.util.*;
import java.net.*;

/**
 * This is a SpandHandler object that fetches the number of
 * Performance Reports for for a given address. Clients can fetch a
 * summary for an address by creating a message with "spand.GetSumm"
 * as the handler and with a SummRequestPayload object as the
 * payload. If the address in the SummRequestPayload is INADDR_ANY
 * (0.0.0.0), then a Summary for each address in the Performance
 * Server's database is returned. In any case, A
 * SendableSpandObjectColl object is returned with the
 * SummResponsePayload objects.
 * @version $Revision: 1.6 $
 * @author SPAND Group, UC Berkeley.  */
public class GetSumm implements SpandHandler {

  public Msg execute (Msg m, Hashtable globalHash){
    Msg reply=null;
    int numReports=0;
    Vector summaries = new Vector();

    Hashtable reportHash=(Hashtable) globalHash.get("spand.ReportPayload");
    /* Make the reportHash if it doesn't yet exist. */
    if (reportHash==null){
      reportHash = new Hashtable();
      globalHash.put(new String("spand.ReportPayload"), reportHash);
    } 

    /* Unmarshal the data portion of the message into a SummRequestPayload object */
    SummRequestPayload req = new SummRequestPayload();

    req.Unmarshal(m.data, 0);

    /* If the request address is INADDR_ANY, then return a summary for
       all known addresses */
    if (req.addr.getHostAddress().compareTo("0.0.0.0")==0){
      /* Loop over entries in the hash table, getting number of
         reports for each address */
      Enumeration keys=reportHash.keys();

      while (keys.hasMoreElements()){
	InetAddress key = (InetAddress) keys.nextElement();
	summaries.addElement(new SummResponsePayload(key, ((Vector) reportHash.get(key)).size()));
      }
      
    } else {
      /* Get the vector of Performance Reports for the specified address */
      Vector reports = (Vector) reportHash.get(req.addr);
      if (reports!=null){
	summaries.addElement(new SummResponsePayload(req.addr, reports.size()));
      }
    }
    
    reply = new Msg("spand.SendableSpandObjectColl",
		      new SendableSpandObjectColl("spand.SummResponsePayload",
						  summaries), false);

    return reply;
  }

  /** Print out the data portion of the specified message */
  public String toString(Msg m){
    SummRequestPayload s = new SummRequestPayload();
    s.Unmarshal(m.data,0);

    return (s.toString());
  }
    
}
