/*
 * Copyright (c) 1996-1998 Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the SPAND Research
 *      Group at the University of California Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
/* $Header: /home/stemm/cvs/JavaSpand/spand/PerfResponsePayload.java,v 1.9 1998/06/10 23:12:29 stemm Exp $ */
package spand;

import java.net.*;

/** A Performance Response Message object, returned by a "spand.GetPerf" handler.
  * @author SPAND Group, UC Berkeley.
 * @version $Revision: 1.9 $.
 */
public class PerfResponsePayload implements SendableSpandObject{

  /** The address that this response deals with */
  public InetAddress addr;

  /** The average available bandwidth from this host, in bits/sec */
  public int availBandwidth;

  /** Standard Deviation of available bandwidth from this host, in bits/sec */
  public int stdDevAvailBandwidth;

  /** The probability that a packet will be lost along the path from this host, in 1/1000s */
  public int packetLossProb;

  /** Standard Deviation of packet loss probability, in 1/1000s */
  public int stdDevPacketLossProb;

  /** The number of Reports used to make this Response */
  public int numReports;

  /** The total number of bytes seen from this host */
  public int numBytes;

  /** The average Srtt from this host, in microseconds */
  public int avgSrtt;

  /** Standard Deviation of srtt */
  public int stdDevSrtt;

  /** The size of the marshalled object */
  final public int wfSize=36;

  /** Constructor that explicitly provides addresss. */
  public PerfResponsePayload(InetAddress addr0){
    this.addr=addr0;
  }

  /** Default constructor */
  public PerfResponsePayload(){
    try {
      this.addr=InetAddress.getByName("0.0.0.0");;
    } catch (UnknownHostException e1){
    }
  }

  /** Unpackage this object from a sequence of bytes. The number of
      bytes read is returned. */
  public int Unmarshal(byte[] wireData, int startIndex) {

    int pos=startIndex;
    
    addr = ArrayConv.ByteArrayToAddr(wireData, pos);
    pos+=4;
    availBandwidth = ArrayConv.ByteArrayToInt(wireData, pos);
    pos+=4;
    stdDevAvailBandwidth = ArrayConv.ByteArrayToInt(wireData, pos);
    pos+=4;
    packetLossProb = ArrayConv.ByteArrayToInt(wireData, pos);
    pos+=4;
    stdDevPacketLossProb = ArrayConv.ByteArrayToInt(wireData, pos);
    pos+=4;
    numReports = ArrayConv.ByteArrayToInt(wireData, pos);
    pos+=4;
    numBytes = ArrayConv.ByteArrayToInt(wireData, pos);
    pos+=4;
    avgSrtt = ArrayConv.ByteArrayToInt(wireData, pos);
    pos+=4;
    stdDevSrtt = ArrayConv.ByteArrayToInt(wireData, pos);
    pos+=4;

    return (pos-startIndex);
  }

  /** Package this object into a sequence of bytes. The number of
      bytes written is returned. If wireData is NULL, then no bytes
      are actually written, but the size of the marshalled object is
      returned. */
  public int Marshal(byte[] wireData, int startIndex) {
    if (wireData==null){
      return wfSize;
    }

    int pos=startIndex;
    
    ArrayConv.AddrToByteArray(wireData, pos, addr);
    pos+=4;
    ArrayConv.IntToByteArray(wireData, pos, availBandwidth);
    pos+=4;
    ArrayConv.IntToByteArray(wireData, pos, stdDevAvailBandwidth);
    pos+=4;
    ArrayConv.IntToByteArray(wireData, pos, packetLossProb);
    pos+=4;
    ArrayConv.IntToByteArray(wireData, pos, stdDevPacketLossProb);
    pos+=4;
    ArrayConv.IntToByteArray(wireData, pos, numReports);
    pos+=4;
    ArrayConv.IntToByteArray(wireData, pos, numBytes);
    pos+=4;    
    ArrayConv.IntToByteArray(wireData, pos, avgSrtt);
    pos+=4;
    ArrayConv.IntToByteArray(wireData, pos, stdDevSrtt);
    pos+=4;
    
    return (pos-startIndex);
  }

  /** Return a String representation of this object. */
  public String toString() {
    return "addr=" + addr.toString() + " availBW=" + availBandwidth + 
      " sdAvailBW=" + stdDevAvailBandwidth + " lossProb=" + 
      packetLossProb + " sdLossProb=" + stdDevPacketLossProb +
      " numReports=" + numReports +
      " numBytes=" + numBytes +
      " avgSrtt=" + avgSrtt +
      " stdDevSrtt=" + stdDevSrtt;
  }
}
