/*
 * Copyright (c) 1996-1998 Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the SPAND Research
 *      Group at the University of California Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
/* $Header: /home/stemm/cvs/JavaSpand/spand/SendableSpandObjectColl.java,v 1.6 1998/06/27 01:41:19 stemm Exp $ */
package spand;
import java.util.*;
import java.lang.*;

/** A collection of SendableSpandObjects
  * @author SPAND Group, UC Berkeley.
 * @version $Revision: 1.6 $.
 */
public class SendableSpandObjectColl implements SendableSpandObject{

  /** The Collection of objects */
  public Vector objects;

  /** The type of the objects in the collection */
  public String name; 

  /** Default constructor */
  SendableSpandObjectColl(){
    objects=new Vector();
    name = new String("none");
  }

  /** Constructor that explicitly provides name and list of objects. */
  SendableSpandObjectColl(String name0, Vector v){
    objects=v;
    name = new String(name0);
  }
  
/** Package this object into a sequence of bytes. The number of
      bytes written is returned. If wireData is NULL, then no bytes
      are actually written, but the size of the marshalled object is
      returned. */
  public int Marshal(byte[] wireData, int startIndex){
    int i;
    int pos=startIndex;
    Enumeration e;
    
    if (wireData==null){
      int size=8+name.length();
      while((size%4)!=0){
  	size++;
      }
      e = objects.elements();
      while (e.hasMoreElements()){
	size+=(((SendableSpandObject) e.nextElement()).Marshal(null, 0));
      }
      return(size);
    }

    /* Write the type of each object */
    ArrayConv.StringToByteArray(wireData, pos, name);
    pos+=(name.length()+4);
    while((pos%4)!=0){
      pos++;
    }
    
    /* Write the number of Objects */
    ArrayConv.IntToByteArray(wireData, pos, objects.size());
    pos+=4;

    /* Loop over objects, writing each */
    e=objects.elements();
    while (e.hasMoreElements()){
      pos+=(((SendableSpandObject) e.nextElement()).Marshal(wireData, pos));
    }

    return (pos-startIndex);
  }
  
/** Unpackage this object from a sequence of bytes. The number of
      bytes read is returned. */
  public int Unmarshal(byte[] wireData, int startIndex){
    int i;
    int pos=startIndex;
    int count;

    /* Read the type of each object */
    name=ArrayConv.ByteArrayToString(wireData, pos);
    pos+=(name.length()+4);
    while((pos%4)!=0){
      pos++;
    }
    
    /* Read the number of Objects, (but don't do anything with it) */
    count=ArrayConv.ByteArrayToInt(wireData, pos);
    pos+=4;

    /* Loop over objects, reading each */
    for(i=0; i<count; i++){
      try {
	SendableSpandObject o = (SendableSpandObject) Class.forName(name).newInstance();
	pos+=o.Unmarshal(wireData, pos);
	objects.addElement(o);
      } catch (ClassNotFoundException e1) {
	System.err.println("Couldn't find class "+name);
	return(pos-startIndex);
      } catch (InstantiationException e2) {
	System.err.println("Couldn't create class "+name);
	return(pos-startIndex);
      } catch (IllegalAccessException e3) {
	System.err.println("Don't have permission to create class "+name);
	return(pos-startIndex);
      }
    }

    return (pos-startIndex);
  }

  /** Display this object as a string */
  public String toString(){
    String temp;
    int i;
    Enumeration e;
    
    temp = new String(name+" "+objects.size());

    e = objects.elements();
    while(e.hasMoreElements()){
      temp = new String(temp+" "+e.nextElement().toString());
    }

    return temp;
    
  }
}

