/*
 * Copyright (c) 1996-1998 Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the SPAND Research
 *      Group at the University of California Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
/* $Header: /home/stemm/cvs/JavaSpand/spand/SpandHandler.java,v 1.5 1998/06/24 21:41:44 stemm Exp $ */
package spand;
import java.util.Hashtable;

/**
 * An object that conforms to the SpandHandler interface can be used
 * as the Handler field of a SPAND Message. In this way, the
 * Performance Server can be made extensible, adding new operations on
 * the database by creating objects that conform to the SpandHandler
 * interface.
 * @version $Revision: 1.5 $
 * @author SPAND Group, UC Berkeley.
 */
    
public interface SpandHandler {
  /**
   * Run the handler, taking the Message and hashtable as arguments.
   * By convention, the Hashtable maps from String Objects to
   * arbitrary Java Objects. The String Objects should contain the
   * full Java class path to the type of Performance Report. The
   * SpandHandler that uses the Performance Report will know what the
   * type of the hashed-to value in the hash table is. For example,
   * SpandHandlers that use the default ReportPayload Performance
   * Reports use a first level key of "spand.ReportPayload", and the
   * value associated with that key in the Hashtable is an additional
   * Hashtable that maps from InetAddress objects to Vectors of
   * ReportPayload objetcs. SpandHandlers that want to add customized
   * Performance Reports should add them by adding additional keys to
   * the top-level Hashtable. */
  public Msg execute(Msg m, Hashtable hash);

  /**
   * Print out a representation of the data for this handler
   */
  public String toString(Msg m);
}
