/*
 * Copyright (c) 1996-1998 Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the SPAND Research
 *      Group at the University of California Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
/* $Header: /home/stemm/cvs/SPAND/javasrc/Snapshot/Snapshot.java,v 1.5 1998/06/25 23:51:11 stemm Exp $ */
import java.awt.*;
import java.applet.*;
import java.util.*;
import java.lang.*;
import java.lang.Math;
import java.io.*;
import java.net.*;
import spand.*;
import graph.*;
import util.*;

/**
 * Snapshot is an applet that can be used to browse the contents of
 * the Performance Server's database. It takes the following parameters:
 <lu>
 <li> HOST: the name of the initial host for which to show performance.
 <li> SPAND_SERVER: the name of the SPAND Performance Server.
 <li> MARKERS: the name of the file that contains marker descriptions. Used by the graphing library
 </lu>
 * @version $Revision: 1.5 $
 * @author SPAND Group, UC Berkeley.
 */
public class Snapshot extends Applet {
  private Font font;
  static Label reportCountLabel, meanLabel, medLabel, sdLabel;
  static PrintGraph timeGraph, cdfGraph;
  Panel listPanel, leftPanel, rightPanel, topLeftPanel, summPanel;
  GridBagLayout gridbag;
  List list;
  DataSet data1, cdfData;
  Axis timeYaxis, timeXaxis, cdfXaxis, cdfYaxis;
  URL markersURL;
  QSortAlgorithm q = new QSortAlgorithm();
  Checkbox update;
  String spandSvr = null;
  
  /** Convinience method */
  public void constrain(Container container, Component component,
			int grid_x, int grid_y, int grid_width, int grid_height,
			int fill, int anchor, double weight_x, double weight_y,
			int top, int left, int bottom, int right)
    {
      GridBagConstraints c = new GridBagConstraints();
      c.gridx = grid_x; c.gridy = grid_y;
      c.gridwidth=grid_width; c.gridheight=grid_height;
      c.fill=fill; c.anchor=anchor;
      c.weightx=weight_x; c.weighty=weight_y;
      if (top+bottom+left+right>0){
	c.insets=new Insets(top, left, bottom, right);
      }
      
      ((GridBagLayout) container.getLayout()).setConstraints(component,c);
      
      container.add(component);
    }
  
  /** Another convenience method that calls full version of constrain() */
  public void constrain(Container container, Component component,
			int grid_x, int grid_y, int grid_width, int grid_height) {
    constrain(container, component, grid_x, grid_y, grid_width, grid_height,
	      GridBagConstraints.NONE, GridBagConstraints.NORTHWEST, 0.0, 0.0,
	      0, 0, 0, 0);
  }
  
  /** Another convenience method that calls full version of constrain() */
  public void constrain(Container container, Component component,
			int grid_x, int grid_y, int grid_width, int grid_height,
			int top, int left, int bottom, int right) {
    constrain(container, component, grid_x, grid_y, grid_width, grid_height,
	      GridBagConstraints.NONE, GridBagConstraints.NORTHWEST, 0.0, 0.0,
	      top, left, bottom, right);
  }
  
  
  /** Initialize the applet. Read and parse parameters and create the
      scrolling list and graphing objects. */
  public void init() {
    List l;
    int i;
    String mfile = getParameter("MARKERS");
    String initialHost = getParameter("HOST");
    spandSvr = getParameter("SPAND_SERVER");
    System.err.println("host="+initialHost);
    
    font = new Font("Helvetica", Font.BOLD, 12);
    setFont(font);
    
    gridbag = new GridBagLayout();
    
    // Add a label that keeps track of the total number of
    // reports received.
    reportCountLabel = new Label("0 Reports      ");
    meanLabel = new Label("Mean BW:              ");
    medLabel = new Label("Median BW:             ");
    sdLabel = new Label("Std Dev BW:             ");
    update = new Checkbox("Allow Updates");
    update.setState(true);
    
    // Add a scrolling list that displays the reports
    // received for each source. This list is sorted by
    // the number of reports recieved.
    list = new List(10, false);
    
    // Add a timeGraph. This timeGraph displays the reports for a
    // single source address.
    timeGraph = new PrintGraph();
    timeGraph.setGraphBackground(new Color(255,255,255));
    timeGraph.borderTop    = 10;
    timeGraph.borderBottom = 10;
    timeGraph.zerocolor = new Color(0,0,0);
    
    // Add a cdfGraph. This displays the cdf of throughputs for that
    // host.
    cdfGraph = new PrintGraph();
    cdfGraph.setGraphBackground(new Color(255,255,255));
    cdfGraph.borderTop    = 10;
    cdfGraph.borderBottom = 10;
    cdfGraph.zerocolor = new Color(0,0,0);
    
    // Read in the markers 
    try {
      markersURL = new URL(getDocumentBase(),mfile);
      timeGraph.setMarkers(new Markers(markersURL));
      cdfGraph.setMarkers(new Markers(markersURL));
    } catch(Exception e) {
      System.out.println("Failed to create Marker URL!");
    }
    
    listPanel = new Panel();
    listPanel.setLayout(gridbag);
    
    topLeftPanel = new Panel();
    topLeftPanel.setLayout(gridbag);
    
    leftPanel = new Panel();
    leftPanel.setLayout(gridbag);
    
    rightPanel = new Panel();
    rightPanel.setLayout(gridbag);
    
    summPanel = new Panel();
    summPanel.setLayout(gridbag);
    
    constrain(listPanel, list, 0,0,1,1,
	      GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER,
	      0.0, 0.0, 0, 0, 0, 0);
    
    // Make a Panel for the three stats labels
    constrain(summPanel, reportCountLabel, 0,0,1,1,
	      GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER,
	      0.0, 0.0, 0, 0, 0, 0);
    constrain(summPanel, meanLabel, 0,1,1,1,
	      GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER,
	      0.0, 0.0, 0, 0, 0, 0);
    constrain(summPanel, medLabel, 0,2,1,1,
	      GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER,
	      0.0, 0.0, 0, 0, 0, 0);
    constrain(summPanel, sdLabel, 0,3,1,1,
	      GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER,
	      0.0, 0.0, 0, 0, 0, 0);
    constrain(summPanel, update, 0,4,1,1,
	      GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER,
	      0.0, 0.0, 0, 0, 0, 0);
    
    constrain(topLeftPanel, listPanel, 0,0,1,1,
	      GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER,
	      0.0, 0.0, 0, 0, 0, 0);
    
    constrain(topLeftPanel, summPanel, 1,0,1,1,
	      GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER,
	      0.0, 0.0, 0, 0, 0, 0);
    
    constrain(leftPanel, topLeftPanel, 0,0,1,1,
	      GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER,
	      0.0, 0.0, 0, 0, 0, 0);
    constrain(leftPanel, new Label("CDF of Performance for Host:"), 0,1,1,1,
	      GridBagConstraints.NONE, GridBagConstraints.CENTER,
	      0.0, 0.0, 0, 0, 0, 0);
    constrain(leftPanel, cdfGraph, 0,2,1,1, GridBagConstraints.BOTH,
	      GridBagConstraints.CENTER,
	      1.0, 1.0, 0, 0, 0, 0);
    
    // Add the time Graph to the top left of the second panel
    constrain(rightPanel, new Label("Performance for Host:"), 0,0,1,1,
	      GridBagConstraints.NONE, GridBagConstraints.CENTER,
	      0.0, 0.0, 0, 0, 0, 0);
    constrain(rightPanel, timeGraph, 0,1,1,1, GridBagConstraints.BOTH,
	      GridBagConstraints.CENTER,
	      1.0, 1.0, 0, 0, 0, 0);
    
    // Use a GridBagLayout to arrange the panels
    this.setLayout(gridbag);
    constrain(this, leftPanel, 0, 0, 1, 1, GridBagConstraints.BOTH,
	      GridBagConstraints.CENTER, 0.5, 1.0, 10, 10, 5, 5);
    constrain(this, rightPanel, 1, 0, 1, 1, GridBagConstraints.BOTH,
	      GridBagConstraints.CENTER, 1.0, 1.0, 10, 10, 5, 10);
    
    // This thread starts a sub-thread. This thread periodically asks
    // the Performance Server for a summary and modifies a scrolling
    // list that contains the addresses and how many reports have been
    // received for each address.
    SnapshotUIModifier t1 = new SnapshotUIModifier(list, reportCountLabel, update, spandSvr);
    t1.start();
    
    // If an initial host is specified, get the performance for that host
    if (initialHost!=null){
      ShowPerfFor(initialHost);
    }
  }  // end init()
  
  
  /** Handle input. If a host in the scrolling list was double
      clicked, then show more information on the performance for that
      host. */
  public boolean action(Event evt, Object arg) {
    /* Synchronize on the scrolling list */
    synchronized (list){
      if (evt.target instanceof List) {
	// Get the Vector of reports for that address
	StringTokenizer tok = new StringTokenizer(arg.toString());
	String name = tok.nextToken();
	//	  System.err.println(name);
	ShowPerfFor(name);
      }
    }
    return true;
  }  // end action()
  
  
  /** Get the Raw Performance Reports for a host, and draw a CDF of
      available bandwidths and the bandwidths as a function of time. */
  private void ShowPerfFor(String name)
    {
      double sum2x;
      double meanBW, medianBW, stdDevBW;
      InetAddress qaddr;
      Requestor requestor;
      
      try{
	if (spandSvr==null){
	  requestor =  new Requestor("spand.cs.berkeley.edu");
	} else {
	  requestor =  new Requestor(spandSvr);
	}
	
	// Detach old data set, if any
	if (data1!=null){
	  timeGraph.detachDataSet(data1);
	  timeGraph.detachAxis(timeXaxis);
	  timeGraph.detachAxis(timeYaxis);
	}
	
	data1 = new DataSet();
	data1.linecolor   = new Color(255,0,0);
	data1.marker      = 1;
	data1.markercolor = new Color(0,0,255);
	timeYaxis = new Axis(Axis.LEFT);
	timeGraph.attachAxis(timeYaxis);
	timeYaxis.attachDataSet(data1);
	timeYaxis.setLabelFont(new Font("Helvetica",Font.PLAIN,10));
	timeYaxis.setTitleText("Observed Throughput (kilobits/sec)");
	timeYaxis.minor_tic_count=0;
	timeXaxis = new Axis(Axis.BOTTOM);
	timeGraph.attachAxis(timeXaxis);
	timeXaxis.attachDataSet(data1);
	timeXaxis.setLabelFont(new Font("Helvetica",Font.PLAIN,10));
	timeXaxis.setTitleText("Time since first report (seconds)");
	
	timeGraph.attachDataSet(data1);
	
	/* Get the collection of reports for this addr */
	try {
	  System.err.println(name);
	  qaddr = InetAddress.getByName(name);
	  Msg replyMsg = requestor.Request(new Msg("spand.GetRaw",
						   new RawRequestPayload(qaddr), true), true);
	  
	  /* Get the Collection of ReportPayload objects */
	  SendableSpandObjectColl reports = (SendableSpandObjectColl) replyMsg.toObject();
	  Enumeration e = reports.objects.elements();
	  
	  // Make a Graph that shows the reported bandwidths as a function of time
	  double startTime=0;
	  
	  // Loop over the reports, adding them to the dataset for the
	  // graph. At the same time, put the throughputs in a vector
	  // (which will be sorted to produce the cdf of throughputs).
	  double throughputs[] = new double[reports.objects.size()];
	  int i=0;
	  while(e.hasMoreElements()){
	    ReportPayload r = (ReportPayload) e.nextElement();
	    double data[]=new double[2];
	    if (i==0){
	      startTime=r.mostSigNTP+r.leastSigNTP/1000000.0;
	    }
	    data[0]=(double) (r.mostSigNTP+r.leastSigNTP/1000000.0)-startTime;
	    throughputs[i]=data[1]= ((double) (r.sampLen*8)/((double) r.sampDur/1000))/1000;
	    
	    //	System.out.println(r.mostSigNTP+" "+data[0]+" "+data[1]);
	    try {
	      data1.append(data,1);
	    } catch (Exception e1){
	      System.out.println("Error appending Data!");
	    }
	    i++;
	  }
	  
	  data1.xaxis.minimum=0.0;
	  data1.yaxis.minimum=0.0;
	  timeGraph.repaint(10);
	  
	  // Sort the list of throughputs
	  try { q.sort(throughputs);} catch (Exception e1) {System.out.println("Couldn't sort throughputs\n");}
	  
	  // Loop through the throughputs, populating another dataset
	  // Detach old data set, if any
	  if (cdfData!=null){
	    cdfGraph.detachDataSet(cdfData);
	    cdfGraph.detachAxis(cdfXaxis);
	    cdfGraph.detachAxis(cdfYaxis);
	  }
	  
	  cdfData = new DataSet();
	  cdfData.linecolor   = new Color(255,0,0);
	  cdfData.marker      = 1;
	  cdfData.markercolor = new Color(0,0,255);
	  cdfYaxis = new Axis(Axis.LEFT);
	  cdfGraph.attachAxis(cdfYaxis);
	  cdfYaxis.attachDataSet(cdfData);
	  cdfYaxis.setLabelFont(new Font("Helvetica",Font.PLAIN,10));
	  cdfYaxis.setTitleText("Cumulative Probability");
	  cdfYaxis.minor_tic_count=0;
	  cdfXaxis = new Axis(Axis.BOTTOM);
	  cdfGraph.attachAxis(cdfXaxis);
	  cdfXaxis.attachDataSet(cdfData);
	  cdfXaxis.setLabelFont(new Font("Helvetica",Font.PLAIN,10));
	  cdfXaxis.setTitleText("Observed Throughput (kilobits/sec)");
	  
	  cdfGraph.attachDataSet(cdfData);
	  
	  sum2x=meanBW=(double) 0.0;
	  
	  for(i=0; i<reports.objects.size(); i++){
	    double data[] = new double[2];
	    data[0]=throughputs[i];
	    data[1]=(double)i/(double)reports.objects.size();
	    
	    try {
	      cdfData.append(data,1);
	    } catch (Exception e1){
	      System.out.println("Error appending Data!");
	    }
	    // Also keep track of the mean, median, and std Dev of bandwidth
	    meanBW += throughputs[i];
	    sum2x += throughputs[i]*throughputs[i];
	  }
	  
	  meanBW/=reports.objects.size();
	  medianBW=throughputs[reports.objects.size()/2];
	  stdDevBW=Math.sqrt(sum2x/reports.objects.size()-meanBW*meanBW);
	  
	  meanLabel.setText("Mean BW: "+meanBW);
	  medLabel.setText("Median BW: "+medianBW);
	  sdLabel.setText("Std Dev BW: "+stdDevBW);
	  
	  cdfData.yaxis.minimum=0.0;
	  cdfData.yaxis.maximum=1.0;
	  cdfGraph.repaint(10);
	} catch (Exception e1){
	  System.err.println("Couldn't get reports for host");
	}
      }
      catch (IOException e1){
	System.err.println("Couldn't make spand requestor");
      }
    }
  
  public String getAppletInfo() {
    return "Used to browse contents of a SPAND Performance Server.";
  }
}   
  
    
