/*
 * Copyright (c) 1996-1998 Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the SPAND Research
 *      Group at the University of California Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
/* $Header: /home/stemm/cvs/Socket++/Channel.h,v 1.4 1998/06/08 15:14:55 ss Exp $ */
#ifndef SS_CHANNEL
#define SS_CHANNEL

#include "SocketTypes.h"
#include "Monitor.h"

/** The Socket++ library provides a simple object oriented interface to 
UNIX network sockets. It also allows the programmer to easily switch
between the use of TCP, UDP and UDP Multicast. 

The Channel class is an abstract class that incorporates the standard
BSD socket routines. All Socket classes derive from this class. */
class Channel {
public:

    Channel();
    /** connect channel to remote channel */
    virtual Sock_Status Connect(char *hostname, Port port) = 0;
    /** listen for connections on port */
    virtual Sock_Status Listen(Port port) = 0;
    /** accept a connection on a port - return val: channel connected to hostname */
    virtual Channel* Accept() = 0;
    /** close channel and connection and then free associated storage */
    virtual Sock_Status Close() = 0;
    /** generic recv will recv at most length bytes */
    virtual Sock_Status Recv(char *buffer, int length) = 0;
    /** generic recv will recv exactly length bytes */
    virtual Sock_Status RecvFull(char *buffer, int length) = 0;
    /** generic send length bytes from buffer */
    virtual Sock_Status Send(char *buffer, int length) = 0;

protected: 
    ChanMonitor *chanMonPtr_;
};

#endif
