/*
 * Copyright (c) 1996-1998 Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the SPAND Research
 *      Group at the University of California Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
/* $Header: /home/stemm/cvs/Socket++/Monitor.cc,v 1.3 1998/05/28 20:15:30 stemm Exp $ */
#include <stdio.h>
#include <string.h>
#include <strings.h>
#include <stdlib.h>
#include <signal.h>
#include <fcntl.h>
#include <errno.h>
#include <assert.h>
#include <sys/file.h>
#include <sys/stat.h>
#include <math.h>
#include "porting.h"
#include "dbgprintlib.h"
#include "Monitor.h"

ChanMonitor::ChanMonitor()
{

    curEpoch_ = (EpochStats *)calloc(sizeof(EpochStats), sizeof(char));
    memset(pastEpochs_, 0, sizeof(pastEpochs_));

    /* Set up state for Network Connection Monitor Stuff */
    bandwidthLessThanFunc_=NULL;
    bandwidthMoreThanFunc_=NULL;
    /* Not Implemented Yet */
#if 0
    LatencyLessThanFunc_=NULL;
    LatencyMoreThanFunc_=NULL;
    PerLessThanFunc_=NULL;
    PerMoreThanFunc_=NULL;
#endif

    /* Set the functions used to compute the cost to Send/Receive Chunks */
    /* BUG: Not actually set. */

    /* Set the functions used to compute the power to Send/Receive Chunks */
    /* BUG: Not actually set. */

    /* Set the functions used to compute the power to Turn the interface On/Off */
    /* BUG: Not actually set. */
}

void
ChanMonitor::Start()
{
    // start taking stats
}

void
ChanMonitor::Stop()
{
    // start taking stats
}

void
ChanMonitor::URecv(int length, timev *nowPtr, int pollFail)
{
    /* if the select poll failed - we issued the recv before data
       arrives - i.e. we are issuing recvs faster than the
       network. Therefore, this recv is part of the same epoch. If the
       poll succeeded, the epoch MIGHT have ended.

       There should be some heuristic to not unnecessarily end
       epochs. For example, a 1KB packet may arrive and the user may
       issue 4 256byte reads. Only the first read will have a select
       fail. Even if the reads are issued at "full" speed. This
       procedure should wait till the next read with a select fail to
       decide when the previous epoch ended. For example, if the
       previous example continues similarly, the first of the next
       four reads will fail also. We can check the time then to decide
       if it is part of the same epoch or part of a different epoch.  */
}

void
ChanMonitor::USend(int length, timev *nowPtr, int pollFail)
{
    /* if the select failed - we issued the send before data cleared out of
       the buffer - i.e. we are issuing sends faster than the network. 
       The remainder of the processing is similar to the recv. */
}
    
#if 0

in timeout code

	else if (temp==0) {
	    debug_printff(102,"Sock_Recv: timeout\n");
			 
	    /* Send a message back to the Sender on the control
	       channel (if one exists) That a timeout occurred. The
	       Sender will use this amount to determine the total
	       bandwidth */
	    if (useCtrl_ == SOCK_USE_CTRL){
		CtrlSendAck(bytesRecvSoFar_, sdPtr->rHost);
	    }
	    sdPtr->bytesRecvSoFar=0;
	    return(SOCK_TIMEOUT);
	}

int Sock_Recv(Socket *sdPtr, char *buffer, int length)
{

    fd_set fdvar;
    int temp,i;
    static double s_i=0, t_i=0, S_i=0, T_i=0, sqrt_sum_squares=0, n=0;
    struct timeval start_tp, end_tp, diff_tp;
    struct timezone start_tzp, end_tzp;
    double b_i, B_i;
	 
    /* Increment the count of number of bytes received so far */
    sdPtr->bytesRecvSoFar += status;

    /* The data was successfully received; Collect Statistics */
    /* We calculate the average bandwidth and compute the standard deviation of the bandwidth as follows: */
    /* Variables: t_i= time taken to complete the ith send */
    /*            s_i= size of the ith send */
    /*            b_i=(s_i)/(t_i) bandwidth for the ith send */
    /*            T_i=t_i+T_(i-1)=total time for first i sends */
    /*            S_i=s_i+S_(i-1)=total size for first i sends */
    /*            B_i=E(b_i)=(S_i/T_i)=Average bandwidth after first i sends */
    /*            Standard Deviation=Sum(sqrt((B_i-b_i)**2))/n */
	 
    gettimeofday(&end_tp, &end_tzp);
    diff_tp.tv_sec=end_tp.tv_sec-start_tp.tv_sec;
    diff_tp.tv_usec=end_tp.tv_usec-start_tp.tv_usec;
    if (diff_tp.tv_usec < 0) {
	diff_tp.tv_usec += 1000000;
	diff_tp.tv_sec -= 1;
    }
	 
    s_i += status;
    t_i +=(diff_tp.tv_sec + (float) diff_tp.tv_usec/1000000.0);
    if (s_i >= 2*sdPtr->lXmitBufSize){

	n++;
	b_i=(s_i*8.)/(t_i)/1000000.0;
	T_i+=t_i;
	S_i+=s_i;
	B_i=S_i*8./T_i/1000000.0;
	sqrt_sum_squares+=sqrt((B_i-b_i)*(B_i-b_i));
		 
	sdPtr->stats.bandwidth=B_i;
	sdPtr->stats.bandwidthAge.tv_sec=end_tp.tv_sec;
	sdPtr->stats.bandwidthAge.tv_usec=end_tp.tv_usec;
	sdPtr->stats.bandwidthStdDev=sqrt_sum_squares/n;
		 
	debug_printff(110, "Sock_Recv: recv %f bytes in %f secs, BW is %f Mb/sec\n", s_i, t_i, sdPtr->stats.bandwidth);
	s_i=0;
	t_i=0;

	/* Check bandwidth, possibly do upcall if bandwidth has changed */
	CHECK_BANDWIDTH(sdPtr);
    }
	 
    return(status);
}

in recv ctrl

    case SOCK_CTRL_ACK:
	debug_printff(109, "recvctrl:Saw ack of %d bytes\n", ctrlPkt.data.ack.numBytes);
	debug_printff(109, "recvctrl:bytesSentSoFar=%d, bytesAckedSoFar=%d\n", sdPtr->bytesSentSoFar, sdPtr->bytesAckedSoFar);

	/* Use this to calculate the bandwidth */
	/* If no bytes left outstanding, then Stop Timer and calculate bandwidth */
	if (sdPtr->bytesSentSoFar == sdPtr->bytesAckedSoFar) {
	    gettimeofday(&end_tp, &end_tzp);
				 
	    t_i = (end_tp.tv_sec*1000000+end_tp.tv_usec)-(sdPtr->start_tp.tv_sec*1000000+sdPtr->start_tp.tv_usec);

	    /*				 sdPtr->stats.bandwidthClosedLoop = (sdPtr->bytesAckedSoFar*8.)/t_i*1000000.;*/
				 
	    sdPtr->bytesSentSoFar=0;
	    sdPtr->bytesAckedSoFar=0;
				 
	    debug_printff(109, "recvctrl:bw est is %f Mb/sec\n", b_i);
	} else {
	    /* Increase the number of bytes that have been acked */	return(status);

	    sdPtr->bytesAckedSoFar += ctrlPkt.data.ack.numBytes;
	}
		 
	break;


/* Send a socket-layer acknowledgement on the control port */
int Sock_CtrlSendAck(Socket *sdPtr, int length, RemoteHost *rhostPtr)
{
    SockCtrlPkt ctrlPkt;
     
    debug_printff(109, "sendctrl:ack ctrl pkt\n");
	 
    ctrlPkt.data.ack.numBytes=length;
    ctrlPkt.reqType = htonl(SOCK_CTRL_ACK);
    return(Sock_SendCtrl(sdPtr, rhostPtr, &ctrlPkt));
}



/*************************************************************
*
* Network Connection Monitor Routines
*
*************************************************************/

void Sock_RegisterBWLessThan(Socket *sdPtr, float newBandwidth, void (*ChangedFunc)(SockStats *))
{
	sdPtr->bandwidthLowWater=newBandwidth;
	sdPtr->bandwidthLessThanFunc=ChangedFunc;
}
void Sock_RegisterBWMoreThan(Socket *sdPtr, float newBandwidth, void (*ChangedFunc)(SockStats *))
{
	sdPtr->bandwidthHighWater=newBandwidth;
	sdPtr->bandwidthMoreThanFunc=ChangedFunc;
}

/* These aren't implemented yet */
#if 0
void Sock_RegisterLatencyLessThan(Socket *sdPtr, float newLatency, void (*ChangedFunc)(SockStats *))
{
}

void Sock_RegisterLatencyMoreThan(Socket *sdPtr, float newLatency, void (*ChangedFunc)(SockStats *))
{
}

void Sock_RegisterPerLessThan(Socket *sdPtr, float newPer, void (*ChangedFunc)(SockStats *))
{
}

void Sock_RegisterPerMoreThan(Socket *sdPtr, float newPer, void (*ChangedFunc)(SockStats *))
{
}
#endif


#endif
