/*
 * Copyright (c) 1996-1998 Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the SPAND Research
 *      Group at the University of California Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
/* $Header: /home/stemm/cvs/Socket++/Monitor.h,v 1.3 1998/06/02 16:45:50 stemm Exp $ */
#ifndef SS_CHANNEL_MONITOR
#define SS_CHANNEL_MONITOR

#include "SocketTypes.h"

typedef struct EpochStats {
    /* Stuff for statistics collection */
    struct timeval startT;	// start time of this epoch 
    struct timeval endT;	// end time of this epoch 

    int rCnt;	 		// bytes received this epoch 
    int sCnt;	 		// bytes sent this epoch 

    int intPktDel;		// avg. int pkt arrival delay in current epoch 
    int intPktDelDev;		// mean dev. of above - like TCP

#if 0
    float rtt;    		// Round-trip latency in milliseconds 
    float rttDev;	   	// deviation in latency 
    float per;    		// Packet error rate 
    float perDev; 	    	// Variance in packet error rate 
#endif	

    int BW;			// BW this epoch

} EpochStats;


/** This class snoops on a Socket object and collects statistics about
    the bandwidth being used by the Socket. A user can also register
    callbacks that will be triggered when the bandwidth changes above
    or below a high/low water mark. This code is incomplete. */
class ChanMonitor {
public:
    ChanMonitor();

    /** Start Monitoring the statistics */
    void Start();

    /** Stop Monitoring the statistics */
    void Stop();
    
    void URecv(int length, timev *nowPtr, int pollFail);
    void USend(int length, timev *nowPtr, int selectFail);
    /* Urtt(int rtt, timev *nowPtr); */
    /* Urtt(int success, timev *nowPtr); */

    /** Invoke the specified callback function if the bandwidth drops below the specified value */
    void RegisterBWLessThan(float BW, void (*ChangedFunc)(EpochStats *));
    /** Invoke the specified callback function if the bandwidth rises above the specified value */
    void RegisterBWMoreThan(float BW, void (*ChangedFunc)(EpochStats *));

    /*
    void RegisterLatencyLessThan(float Lat, void (*ChangedFunc)(ChanStats *));
    void RegisterLatencyMoreThan(float Lat, void (*ChangedFunc)(ChanStats *));
    void RegisterPerLessThan(float Per, void (*ChangedFunc)(ChanStats *));
    void RegisterPerMoreThan(float Per, void (*ChangedFunc)(ChanStats *));
    */


protected:
#define NUM_PAST_EPOCHS 4
    EpochStats *curEpoch_, *pastEpochs_[NUM_PAST_EPOCHS];

    /** The function that is called when the bandwidth tatistics
    change enough to trigger a registration */
    void (*bandwidthLessThanFunc_)(EpochStats *);
    
    /** If the bandwidth decreases to less than this, call the
        BandwidthLessThan Function */
    float bandwidthLowWater_;
    
    /** The function that is called when the bandwidth statistics
       change enough to trigger a registration */
    void (*bandwidthMoreThanFunc_)();
    
    /** If the bandwidth increases more than this, call the
       BandwidthMoreThan Function */
    float bandwidthHighWater_;
    
    /* These aren't implemented yet */
#if 0	 
    /* The function that is called when the latency */
    /* statistics change enough to trigger a registration */
    void (*LatencyLessThanFunc_);
    /* The function that is called when the latency */
    /* statistics change enough to trigger a registration */
    void * (*LatencyMoreThanFunc_);
    /* The function that is called when the per */
    /* statistics change enough to trigger a registration */
    void (*PerLessThanFunc_);
    /* The function that is called when the per */
    /* statistics change enough to trigger a registration */
    void (*PerMoreThanFunc_);
#endif	 
};



/* Macro to check bandwidths and possibly call bandwidthChanged functions */
#define CHECK_BANDWIDTH(sdPtr) {\
  if (sdPtr->stats.bandwidth < sdPtr->bandwidthLowWater && \
      sdPtr->bandwidthLessThanFunc!=NULL)\
  { \
	sdPtr->bandwidthLessThanFunc(&(sdPtr->stats));\
  } else \
	if (sdPtr->stats.bandwidth>sdPtr->bandwidthHighWater && \
	    sdPtr->bandwidthMoreThanFunc!=NULL) \
	{ \
	      sdPtr->bandwidthMoreThanFunc(&(sdPtr->stats)); \
	} \
  }

#endif
