/*
 * Copyright (c) 1996-1998 Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the SPAND Research
 *      Group at the University of California Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
/* $Header: /home/stemm/cvs/Socket++/RelMulti.cc,v 1.4 1998/06/08 15:14:55 ss Exp $ */

#if 0

#include "porting.h"

#ifdef MULTICAST
#include <stdio.h>
#include <string.h>
#include <strings.h>
#include <stdlib.h>
#include <signal.h>
#include <fcntl.h>
#include <errno.h>
#include <assert.h>
#include <sys/file.h>
#include <sys/stat.h>
#include <math.h>
#include "dbgprintlib.h"
#include "Socket.h"
#include "Udp.h"
#include "UdpMulti.h"


UdpMultiSocket::UdpMultiSocket() : UdpSocket(SOCK_NOT_CTRL)
{
    if (InitSocket() != SOCK_SUCCESS) {
	state_ = SOCK_STATE_CLOSED;
	return;
    }
}

UdpMultiSocket::UdpMultiSocket(int isCtrl = SOCK_NOT_CTRL) : UdpSocket(isCtrl)
{
    if (InitSocket() != SOCK_SUCCESS) {
	state_ = SOCK_STATE_CLOSED;
	return;
    }
}
    
Sock_Status
UdpMultiSocket::InitSocket()
{
    Sock_Status status;

    status = UdpSocket::InitSocket();
    protocol_ = SOCK_UDP_MULTI;
    ttl_ = SOCK_DEFAULT_TTL;
    return status;
}

Sock_Status
UdpMultiSocket::Connect(char *hostname, Port port)
{
    u_char c;

    Socket::Connect(hostname, port);

    if (IN_CLASSD(ntohl(rSin_.sin_addr.s_addr))) {
	struct ip_mreq mr;

	/*
	 * Try to bind the multicast address as the socket
	 * dest address.  On many systems this won't work
	 * so fall back to a destination of INADDR_ANY if
	 * the first bind fails.
	 */       
	lSin_.sin_family = AF_INET;
	lSin_.sin_port = htons(port);
	lSin_.sin_addr.s_addr = rSin_.sin_addr.s_addr;
	if ((bind_(sfd_, (struct sockaddr *)&lSin_, sizeof(lSin_))) == -1) {
	    lSin_.sin_addr.s_addr = htonl(INADDR_ANY);
	    if ((bind_(sfd_, (struct sockaddr *)&lSin_, sizeof(lSin_))) == -1){
		debug_printf(110, "Sock_connect: error binding socket");
		return(SOCK_FAILURE);
	    }
	}
	/* 
	 * XXX This is bogus multicast setup that really
	 * shouldn't have to be done (group membership should be
	 * implicit in the IP class D address, route should contain
	 * ttl & no loopback flag, etc.).  Steve Deering has promised
	 * to fix this for the 4.4bsd release.	       
	 */
	mr.imr_multiaddr.s_addr = rSin_.sin_addr.s_addr;
	mr.imr_interface.s_addr = INADDR_ANY;
	if (setsockopt(sfd_, IPPROTO_IP, IP_ADD_MEMBERSHIP, 
		       (char *)&mr, sizeof(mr)) < 0) {
	    debug_printf(110, "sock_connect: IP_ADD_MEMBERSHIP");
	    return(SOCK_FAILURE);
	}
	if (setsockopt(sfd_,IPPROTO_IP, IP_MULTICAST_TTL, (char *)&ttl_, 
		       sizeof(ttl_))<0)
	    debug_perror(10, "Sock_Connect: ip_multicast_ttl");
	c = 0;
	if (setsockopt(sfd_, IPPROTO_IP, IP_MULTICAST_LOOP, (char *)&c, 
		       sizeof(c)) < 0)
	    debug_perror(10, "Sock_Connect: ip_multicast_loop");
	if (compatible_ == SOCK_NOT_COMPATIBLE && isCtrl_ != SOCK_CTRL) {
	    SockCtrlPkt temp;
	    rHost_->udpCtrlPort=port+1;
	    rHost_->tcpCtrlPort=-1;
	    rHost_->tcpCtrl=NULL;
	    rHost_->next=NULL;
	    rHost_->remoteName="MULTICAST";
	    Sock_NameToSin(hostname, rHost_->udpCtrlPort, &(rHost_->udpRSin));
	    /* send "Hello" packet on control channel */
	    strcpy(temp.data.hello.ipAddr, localIPAddr_);
	    temp.data.hello.tcpCtrlPort = htons(GetTCPCtrlPort());
	    temp.data.hello.udpCtrlPort = htons(GetUDPCtrlPort());
	    SendCtrl(rHost_, &temp);
	    /* no welcome/accepted pkt */
	}
	numActiveRHost_=1;
	SetSocketInfo();
	state_ = SOCK_STATE_CONNECTED;
	return(SOCK_SUCCESS);
    } else
	return(SOCK_FAILURE);
}


#endif MULTICAST
#endif 0
