/*
 * Copyright (c) 1996-1998 Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the SPAND Research
 *      Group at the University of California Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
/* $Header: /home/stemm/cvs/Socket++/Socket.cc,v 1.11 1998/06/04 20:39:08 ss Exp $ */

#include <string.h>
#include <strings.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <errno.h>
#include <assert.h>
#include <sys/file.h>
#include <sys/stat.h>
#include <math.h>
#include <unistd.h>
#include "porting.h"
#include "dbgprintlib.h"
#include "Socket.h"
#include <Classes.h>

#define DO_SELECT(numSockets, arg1, arg2, arg3, timeout) \
{ \
  if ((rc=select((numSockets), (arg1), (arg2), (arg3), (timeout)))==0){ \
      return NULL; \
  } else if (rc==-1){ \
    perror("In Select()"); \
  } \
}

/* Given a variable number of socket pointers, returns the first one that has pending data */
/* Returns NULL if timeout expires or error occurs */
/* readWriteExec is concatenation of letters r,w,x */
/* (If a letter is there, then all socks are checked for that condition) */
/* Timeout is in microseconds */
Socket *Select(char *readWriteExec, int usec, Collection<Socket *> &socks)
{
  fd_set fds;
  int read=0, write=0, exec=0;
  int i;
  int rc;
  Socket *temp;
  Socket **tempp;
  int numSockets=0;
  struct timeval timeout;
  struct timeval *timeoutp = &timeout;
  
  if (usec==-1){
	  timeoutp=NULL;
  }
  timeout.tv_sec=usec/1000000;
  timeout.tv_usec=usec%1000000;

  FD_ZERO(&fds);

  for (tempp=socks.First(); tempp!=NULL; numSockets++, tempp=socks.Next()){
    temp=*tempp;
    FD_SET(temp->GetFD(), &fds);
  }

  /* 7 combinations here (r, w, x may be on or off) */
  if (strchr(readWriteExec, 'r')) read=1;
  if (strchr(readWriteExec, 'w')) write=1;
  if (strchr(readWriteExec, 'x')) exec=1;

  if (read && !write && !exec){
    DO_SELECT(FD_SETSIZE, &fds, NULL, NULL, timeoutp);
  } else if (!read && write && !exec){
    DO_SELECT(FD_SETSIZE, NULL, &fds, NULL, timeoutp);
  } else if (!read && !write && exec){
    DO_SELECT(FD_SETSIZE, NULL, NULL, &fds, timeoutp);
  } else if (read && write && !exec){
    DO_SELECT(FD_SETSIZE, &fds, &fds, NULL, timeoutp);
  } else if (!read && write && exec){
    DO_SELECT(FD_SETSIZE, NULL, &fds, &fds, timeoutp);
  } else if (read && !write && exec){
    DO_SELECT(FD_SETSIZE, &fds, NULL, &fds, timeoutp);
  } else if (read && write && exec){
    DO_SELECT(FD_SETSIZE, &fds, &fds, &fds, timeoutp);
  } else {
    fprintf(stderr, "Didn't select on read/write/exec??\n");
  }

  /* If here, no error and at least one Socket has something waiting */
  for (tempp=socks.First(); tempp!=NULL; tempp=socks.Next()){
    temp=*tempp;
    if (FD_ISSET(temp->GetFD(), &fds)){
      return temp;
    }
  }

  /* If here, then select() says a socket is ready but we didn't find it */

  fprintf(stderr, "Didn't find sock that select() said was ready?\n");

  return NULL;
}

int 
Socket::RecvStream(char *buffer, int length)
{
    int temp;

    temp = read(rfd_, buffer, length);
    if (temp == 0) {
	if (errno == EACCES || errno == 0)
	    return(SOCK_FD_CLOSED);
	return(SOCK_FAILURE);
    }
    if (temp == -1)
	debug_printf(101,"Sock_recv: error in recv\n");
    if (temp != length)
	debug_printf(109, "Sock_Recv: warning short read %d read %d wanted\n",
		temp, length);
    return(temp);
}

int 
Socket::RecvStreamFull(register char *buffer, int length)
{
    register int     count = 0;
    register int          nread;
    int temp;

    do {
	if ((temp = WaitForPktRecv()) != SOCK_SUCCESS)
	    return (temp);
	nread = read(rfd_, buffer, length-count);
	if(nread < 0)  {
	    debug_printf(101,"Sock_RecvFull: error in read\n");
	    return(SOCK_FAILURE);
	}
	if(nread == 0) {
	    if (errno == EACCES)
		return(SOCK_FD_CLOSED);
	    return(SOCK_FAILURE);
	}
	count += (unsigned)nread;
	buffer += nread;
    } while(count < length);

    return((int)count);
}

int 
Socket::RecvDgram(char *buffer, int length)
{
    int temp;
    size_t addrLen;

    addrLen = sizeof(lastRSin_);
    temp = recvfrom(rfd_, buffer, length, 0,
		    (struct sockaddr *)&(lastRSin_),
#if defined (LINUX)		    
		    (unsigned int *) &addrLen);
#elif defined(AIX4)
		    (long unsigned int *) &addrLen);
#else
		    (int *) &addrLen);
#endif    
    debug_printf(101, "lastRSin_=%x\n", lastRSin_);
    if (temp == -1) {
	debug_printf(101,"Sock_RecvDgram: bad recvfrom\n");
	return(SOCK_FAILURE);
    }
    if (temp != length)
	debug_printf(109, "Sock_RecvDgram: warning short read %d read %d wanted\n",
		temp, length);
    return(temp);
}

int 
Socket::RecvDgramFull(char *buffer, int length)
{
    size_t addrLen;
    register int     count = 0;
    register int          nread;
    int temp;

    debug_printf(102, "recvdgramfull %d\n", length);
     
    do {
	if ((temp = WaitForPktRecv()) != SOCK_SUCCESS)
	    return (temp);
	addrLen = sizeof(lastRSin_);
	nread=recvfrom(rfd_,buffer,length,0, (struct sockaddr *)&(lastRSin_),
#if (defined LINUX)	  
		       (unsigned int *) &addrLen);
#elif defined (AIX4)
  		       (long unsigned int *) &addrLen);
#else
		       (int *) &addrLen);
#endif
	if (nread == -1) {
	    debug_printf(10, "Sock_RecvDgramFull: bad recvfrom");
	    return(SOCK_FAILURE);
	}
	if (nread == 0)
	    return((int)count);
	count += (unsigned)nread;
	buffer += nread;
    } while(count < length);

    return((int)count);
}

int 
Socket::SendStream(char *buffer, int length)
{
    int temp;
    if ((temp = send(sfd_, buffer, length, lFlags_)) == -1) {
	if (errno == EWOULDBLOCK)
	    return(SOCK_FAILURE);
	debug_printf(110, "Sock_send: error in send\n");
	if (errno == EPIPE) {
	    state_ = SOCK_STATE_CLOSED;
	    return(SOCK_FD_CLOSED);
	}
	return(SOCK_FAILURE);
    }
    return(temp);
}

/* send length bytes from buffer to rsin (faster) */
int 
Socket::SendDgram(char *buffer, int length)
{
    int temp;
    fd_set fdvar;

    FD_SET(sfd_, &fdvar);
    select_(FD_SETSIZE, (fd_set *)0,  &fdvar, (fd_set *)0, NULL);
    while (1) {
	temp = sendto(sfd_, buffer, length, lFlags_,
		      (struct sockaddr *)&(rSin_), 
		      sizeof(rSin_));
	if (temp < 0) {
	    if (errno == ENOBUFS)
		continue;
	    debug_printf(110, "Sock_SendDgram: error %d in sendto\n", errno);
	    if (errno == EPIPE) {
		state_ = SOCK_STATE_CLOSED;
		return(SOCK_FD_CLOSED);
	    }
	    return(SOCK_FAILURE);
	}
	return temp;
    }
}


int 
Socket::WaitForPktSend(int length)
{
     if (useCtrl_ == SOCK_USE_CTRL)
	  CheckCtrl();

     if (state_ == SOCK_STATE_CLOSED || state_ == SOCK_STATE_CLOSING)
               return(SOCK_FD_CLOSED);

     if (xmitRate_ > 0)
	 lastPktSize_ = 
	     Sock_DelayPacket(&lastPktTime_, lastPktSize_, xmitRate_, length);
     return(SOCK_SUCCESS);
}

int 
Socket::WaitForPktRecv()
{
    static struct timeval defltTout = 
    {SOCK_CTRL_TIMEOUT_SEC, SOCK_CTRL_TIMEOUT_USEC};
    fd_set fdvar;
    int temp;

    while (1) {
	if (useCtrl_ == SOCK_USE_CTRL)
	    CheckCtrl();
		 
	if (state_ == SOCK_STATE_CLOSED || 
	    state_ == SOCK_STATE_CLOSING)
	    return(SOCK_FD_CLOSED);

	FD_ZERO(&fdvar);
	FD_SET(rfd_, &fdvar);
		 
	if (block_ == SOCK_NON_BLOCK)
	    return(SOCK_SUCCESS);
		 
	if (toutPtr_ == NULL)
	    temp = select_(FD_SETSIZE, &fdvar, 
			  (fd_set *)0, (fd_set *)0,&defltTout);
	else
	    temp = select_(FD_SETSIZE, &fdvar, 
			  (fd_set *)0, (fd_set *)0,toutPtr_);
	if (temp > 0)
	    return(SOCK_SUCCESS);
	else if (temp==0 && toutPtr_ != NULL) {
	    debug_printf(102,"Sock_Recv: timeout\n");
	    return(SOCK_TIMEOUT);
	}
	
    }
}

Sock_Status
Socket::SetSocketInfo()
{
    int length;

    length = sizeof(Address);
    if (getsockname_(rfd_, (struct sockaddr *)&lSin_, &length) == -1) {
	debug_printf(110, "Sock_Connect: error getting socket name");
	return(SOCK_FAILURE);
    }
    localPort_ = ntohs(lSin_.sin_port);
    debug_printf(109,"connected from inet port %d to %d tcp, %d udp\n",
	    localPort_, rHost_->tcpCtrlPort, rHost_->udpCtrlPort );
    return(SOCK_SUCCESS);
}

int 
Socket::SetRemoteByName(char *hostname, Port port)
{
    /* get remote host address */
    debug_printf(109,"setting remote to %s %d\n", hostname, port);
    Sock_NameToSin(hostname, port, &(rSin_));
    /* set up socket descriptor */
    if (rHost_->remoteName == NULL)
	rHost_->remoteName = (char *) Sock_Malloc(SOCK_MAXHOSTNAME);
    strncpy(rHost_->remoteName, hostname, SOCK_MAXHOSTNAME);
    remotePort_ = port;
    return(SOCK_SUCCESS);
}

int 
Socket::SetRemoteByAddr(Address *rSinPtr)
{
    if (rSinPtr != &(rSin_))
	memcpy(&(rSin_), rSinPtr, sizeof(rSin_));
    if (rHost_->remoteName == NULL)
	rHost_->remoteName = (char *) Sock_Malloc(SOCK_MAXHOSTNAME);
    if (resolve_)
	Sock_SinToName(rHost_->remoteName, SOCK_MAXHOSTNAME, 
		       &(remotePort_), rSinPtr);
    else {
	strncpy(rHost_->remoteName, inet_ntoa(rSinPtr->sin_addr), 
		(size_t) SOCK_MAXHOSTNAME);
	remotePort_ = rSinPtr->sin_port;
    }
    return(SOCK_SUCCESS);
}

int 
Socket::SendHello()
{
    SockCtrlPkt temp;

    /* send "Hello connect" packet */
    strcpy(temp.data.hello.ipAddr, localIPAddr_);
    temp.data.hello.tcpCtrlPort = htons(GetTCPCtrlPort());
    temp.data.hello.udpCtrlPort = htons(GetUDPCtrlPort());
    if (Send((char *)&temp, sizeof(temp)) == SOCK_FAILURE) {
	debug_printf(101, "error sending, connect failed\n");
	return SOCK_FAILURE;
    }
    return SOCK_SUCCESS;
}

int 
Socket::RecvHello()
{
    SockCtrlPkt temp;

    /* receive "Hello accept" packet */
    if (RecvFull((char *)&temp, sizeof(temp)) == SOCK_FAILURE) {
	debug_printf(101, "error recv hello, connect failed\n");
	return SOCK_FAILURE;
    }	    
    rHost_->udpCtrlPort=ntohs(temp.data.hello.udpCtrlPort);
    rHost_->tcpCtrlPort=ntohs(temp.data.hello.tcpCtrlPort);
    rHost_->tcpCtrl=NULL;
    rHost_->next=NULL;
    strcpy(rHost_->remoteIPAddr, temp.data.hello.ipAddr);
    Sock_NameToSin(rHost_->remoteIPAddr, 
		   rHost_->udpCtrlPort,
		   &(rHost_->udpRSin));
    return SOCK_SUCCESS;
}

/*************************************************************************
			 Control Socket Routines
*************************************************************************/

/* recv a ctrl packet on ctrlsdptr and apply operation on sdptr */

void
Socket::CheckCtrl()
{
    RemoteHost *rhPtr;
    fd_set fdvar;
    static struct timeval poll = {0,0};
    int temp;

    rhPtr = rHost_;
    FD_ZERO(&fdvar);
    while (rhPtr != NULL) {
	if (rhPtr->tcpCtrl != NULL)
	    FD_SET(rhPtr->tcpCtrl->rfd_, &fdvar);
	rhPtr = rhPtr->next;
    }
    FD_SET(udpCtrlChannel_->rfd_, &fdvar);
    FD_SET(tcpCtrlChannel_->rfd_, &fdvar);
    if ((temp = select_(FD_SETSIZE, &fdvar, (fd_set *)0, (fd_set *)0,
		       &poll)) <= 0) {
	debug_printf(120,"Sock_Recv: no ctrl info\n");
    }
    else {
	debug_printf(112,"Sock_Recv: ctrl info temp = %d\n",temp);
	if (FD_ISSET(udpCtrlChannel_->rfd_, &fdvar)) {
	    SockCtrlPkt ctrlPkt;

	    if (udpCtrlChannel_->RecvCtrl(&ctrlPkt) == SOCK_SUCCESS)
		ProcessCtrlPkt(&ctrlPkt);
	}
	if (FD_ISSET(tcpCtrlChannel_->rfd_, &fdvar))
	    /* XXXX */
	    debug_printf(120,"Sock_Recv: accept new tcp ctrl\n");
	rhPtr = rHost_;
	while (rhPtr != NULL) {
	    if (rhPtr->tcpCtrl != NULL)
		if (FD_ISSET(rhPtr->tcpCtrl->rfd_, &fdvar)) {
		    SockCtrlPkt ctrlPkt;

		    if (rhPtr->tcpCtrl->RecvCtrl(&ctrlPkt) == SOCK_SUCCESS)
			ProcessCtrlPkt(&ctrlPkt);
		}
	    rhPtr = rhPtr->next;
	}
    }
}

Sock_Status
Socket::RecvCtrl(SockCtrlPkt *ctrlPktPtr)
{
					  
    int status=SOCK_SUCCESS;
    int temp;
    size_t addrLen;
    Address rSin;
	 
    debug_printf(109, "Sock_RecvCtrl\n");
    addrLen = sizeof(rSin);
    temp = recvfrom(rfd_,(char *)ctrlPktPtr,sizeof(SockCtrlPkt),0,
#if (defined LINUX)		    
		    (struct sockaddr *)&(rSin), (unsigned int *) &addrLen);
#elif defined (AIX4)
		    (struct sockaddr *)&(rSin), (long unsigned int *)&addrLen);
#else
		    (struct sockaddr *)&(rSin), (int *) &addrLen);
#endif    
    if (temp==-1) {
	debug_printf(109,"recvctrl:recv failed\n");
	return(SOCK_FAILURE);
    }
    return(status);
}

int 
Socket::SendCtrl(RemoteHost *rhostPtr, SockCtrlPkt *ctrlPktPtr)
{
     
    if (rhostPtr->tcpCtrl != NULL)
	rhostPtr->tcpCtrl->Send((char *) ctrlPktPtr, sizeof(*ctrlPktPtr));
    else {
	if (sendto(udpCtrlChannel_->sfd_, (char *)ctrlPktPtr, 
		   sizeof(*ctrlPktPtr), 0,
		   (struct sockaddr *)&(rhostPtr->udpRSin),
		   sizeof(rhostPtr->udpRSin)) == -1) {
	    debug_printf(110, "Sock_SendCtrl: error in sendto");
	    return(SOCK_FAILURE);
	}
    }
    return(SOCK_SUCCESS);
}

int 
Socket::CtrlSendFarewell(RemoteHost *rhostPtr)
{
    SockCtrlPkt ctrlPkt;
     
    debug_printf(109, "sendctrl:farewell ctrl pkt\n");
    strcpy(ctrlPkt.data.farewell.ipAddr, localIPAddr_);
    ctrlPkt.reqType = htonl(SOCK_CTRL_FAREWELL);
    return(SendCtrl(rhostPtr, &ctrlPkt));
}
			       
void 
Socket::ProcessCtrlPkt(SockCtrlPkt *ctrlPktPtr)
{

    switch (ntohl(ctrlPktPtr->reqType)) {

    case SOCK_CTRL_FAREWELL:
	debug_printf(109, "recvctrl:farewell ctrl pkt\n");
	state_ = SOCK_STATE_CLOSING;
	break;
    case SOCK_CTRL_HELLO:
	debug_printf(109, "recvctrl:hello ctrl pkt\n");
	debug_printf(109, "Hello from %s at udp %d tcp %d\n", 
		ctrlPktPtr->data.hello.ipAddr, 
		ctrlPktPtr->data.hello.udpCtrlPort, 
		ctrlPktPtr->data.hello.tcpCtrlPort);
	/* search and add to rhost list XXXX */
	break;
    case SOCK_CTRL_SET_RATE:
	/* tested */
	xmitRate_ = ctrlPktPtr->data.setRate.rate;
	debug_printf(109, "recvctrl:set rate ctrl pkt rate %d\n", xmitRate_);
	break;
    case SOCK_CTRL_SET_REMOTE:
	/* tested */
	debug_printf(109, "recvctrl:set remote ctrl pkt\n");
	memcpy(&(rSin_), &rSin_, sizeof(Address));
	SetRemoteByAddr(&rSin_);
	break;
    case SOCK_CTRL_NEW_CTRL_CHANNEL:
	/* tested 
	   debug_printf(109, "recvctrl:new ctrl channel\n");
	   for (i=SOCK_MAX_MULTICAST_HOSTS;
	   i<(SOCK_MAX_MULTICAST_HOSTS+SOCK_MAX_CTRL_HOSTS); i++) {
	   if (!activeHosts_[i])
	   break;
	   }
	   ctrlChannel_[i] = Sock_Socket(ctrlPktPtr->data.newCtrlChannel.protocol);
	   status = Sock_Connect(ctrlChannel_[i],
	   ctrlPktPtr->data.newCtrlChannel.hostname,
	   ctrlPktPtr->data.newCtrlChannel.port);
	   activeHosts_[i] = 1;
	   break;
	   */
    default:
	debug_printf(109,"unrecognized ctrl pkt: type %d\n",ctrlPktPtr->reqType);
    }
}

/*************************************************************************
  get/set routines
  *************************************************************************/

int 
Socket::SetTimeout(Usec value)
{
    if (value == SOCK_NO_TIMEOUT) {
	toutPtr_ = NULL;
	debug_printf(109, "no timeouts\n");
    }
    else {
	toutPtr_ = &(timeout_);
	timeout_.tv_sec = value/1000000;
	timeout_.tv_usec = value - (timeout_.tv_sec * 1000000);
	debug_printf(109, "timeout set to %d sec %d usec\n",
		timeout_.tv_sec, timeout_.tv_usec );
    }
    return(value);
}

int 
Socket::SetBufferSize(int value)
{
    int k=0,l,m=0,n;

    l = sizeof(k);
    n = sizeof(m);
    setsockopt(sfd_,SOL_SOCKET,SO_SNDBUF,(char*)&value, sizeof(value));
#if defined(AIX4)
    getsockopt(sfd_,SOL_SOCKET,SO_SNDBUF,(char*)&k,&l);
#else
    getsockopt(sfd_,SOL_SOCKET,SO_SNDBUF,(char*)&k,(unsigned int *) &l);
#endif
    setsockopt(sfd_,SOL_SOCKET,SO_RCVBUF,(char*)&value, sizeof(value));
#if defined(AIX4)
    getsockopt(sfd_,SOL_SOCKET,SO_RCVBUF,(char*)&m, &n);
#else
    getsockopt(sfd_,SOL_SOCKET,SO_RCVBUF,(char*)&m,(unsigned int *) &n);
#endif
    if (sfd_ != rfd_) {
	setsockopt(rfd_,SOL_SOCKET,SO_SNDBUF,(char*)&value, sizeof(value));
#if defined(AIX4)
	getsockopt(rfd_,SOL_SOCKET,SO_SNDBUF,(char*)&k,&l);
#else
	getsockopt(rfd_,SOL_SOCKET,SO_SNDBUF,(char*)&k,(unsigned int *) &l);
#endif
	setsockopt(rfd_,SOL_SOCKET,SO_RCVBUF,(char*)&value, sizeof(value));
#if defined(AIX4)
	getsockopt(rfd_,SOL_SOCKET,SO_RCVBUF,(char*)&m,&n);
#else
	getsockopt(rfd_,SOL_SOCKET,SO_RCVBUF,(char*)&m,(unsigned int *) &n);
#endif
    }
    debug_printf(109, "Sock_setbuffersize: send buf %d, recv buf %d\n", k, m);
    lRecvBufSize_ = m;
    lXmitBufSize_ = k;
    if ((k == value) && (m == value))
	return SOCK_SUCCESS;
    else
	return SOCK_FAILURE;
}

int 
Socket::SetFlag(int flag, int value) 
{
    int val, status = SOCK_SUCCESS;

    val = fcntl(rfd_, F_GETFL, 0);
    if (val < 0)
	status = SOCK_FAILURE;
    else {
	if (value)
	    val |= flag;
	else
	    val &= ~flag;
	if (fcntl(rfd_, F_SETFL, val) < 0)
	    status = SOCK_FAILURE;
    }
    if (rfd_ == sfd_)
	return(status);
    val = fcntl(sfd_, F_GETFL, 0);
    if (val < 0)
	status = SOCK_FAILURE;
    else {
	if (value)
	    val |= flag;
	else
	    val &= ~flag;
	if (fcntl(sfd_, F_SETFL, val) < 0)
	    status = SOCK_FAILURE;
    }
    return(status);
}

int 
Socket::GetFlag(int flag) 
{
    int val;
    val = fcntl(rfd_, F_GETFL, 0);
    if (val < 0)
	return SOCK_FAILURE;
    return(val & flag);
}

int 
Socket::SetSockOpt(int flag, int value, char *layer)
{
    int j = sizeof(value), status = SOCK_SUCCESS;;
    struct protoent *proto = getprotobyname(layer);


    if (setsockopt(rfd_,proto->p_proto, flag, (char *)&value, j) < 0)
	status = SOCK_FAILURE;
    if (rfd_ != sfd_)
	if (setsockopt(sfd_,proto->p_proto, flag, (char *)&value, j) < 0)
	    status = SOCK_FAILURE;
    if (status == SOCK_FAILURE)
	debug_printf(110, "Socket: set sock opt");
    return(status);
}

int 
Socket::GetSockOpt(int flag, char *layer)
{
    int i,j = sizeof(i);
    struct protoent *proto = getprotobyname(layer);
    i=1;

#if defined(AIX4)
    if (getsockopt(rfd_,proto->p_proto, flag, (char *)&i, &j) < 0) {
#else
    if (getsockopt(rfd_,proto->p_proto, flag, (char *)&i, (unsigned int *) &j) < 0) {
#endif
	debug_printf(110, "Socket: get sock opt\n");
	return SOCK_FAILURE;
    }
    return(i);
}

char *
Socket::GetRemoteName()
{
     char *retStr;

     /*     retStr = (char *)malloc(strlen(rHost_->remoteName));
     strcpy(retStr,rHost_->remoteName); */
     retStr = rHost_->remoteName;
     return(retStr);
}

char *
Socket::GetLocalName()
{
     char *retStr;

     /*     retStr = (char *)malloc(strlen(localName_));
     strcpy(retStr,localName_); */
     retStr = localName_;
     return(retStr);
}

Address *
Socket::GetRemoteAddr()
{
    /*    Address *addr = (Address *)malloc(sizeof(Address));
    memcpy(addr, &(rSin_), sizeof(Address)); 
    return(addr);*/
    return(&rSin_);
}

Address *
Socket::GetLocalAddr()
{
    /*    Address *addr = (Address *)malloc(sizeof(Address));
    memcpy(addr, &(lSin_), sizeof(Address));
    return(addr); */
    return(&lSin_);
}

Address *
Socket::GetLastSenderAddr()
{
    /*    Address *addr = (Address *)malloc(sizeof(Address));
    memcpy(addr, &(lastRSin_), sizeof(Address));
    return(addr); */
    return(&lastRSin_);
}


/*************************************************************************
  				Socket Routines
*************************************************************************/

Socket::Socket(int isCtrl = SOCK_NOT_CTRL)
{
    debug_printf(112, "Sock_Socket\n");
    isCtrl_ = isCtrl;
    useCtrl_ = SOCK_DONT_USE_CTRL;
    remotePort_ = 0;
    gethostname(localName_, SOCK_MAX_HOST_NAME_LEN);
    SetTimeout(SOCK_NO_TIMEOUT);
    gettimeofday(&(lastPktTime_), NULL);
    lastPktSize_ = 0;
    xmitRate_ = 0;
    localPort_ = 0;
    resolve_ = 1;
    block_ = SOCK_BLOCK;
    broadcast_ = SOCK_NOT_BROADCAST;
    compatible_ = SOCK_NOT_COMPATIBLE;
    udpCtrlChannel_ = NULL;
    tcpCtrlChannel_ = NULL;
    numActiveRHost_ = 0;
    acceptFlag_ = 0;
    rHost_ = NULL;
    lFlags_ = 0;
    memset(&rSin_, 0, sizeof(rSin_));
    memset(&lSin_, 0, sizeof(lSin_));
    /*    berState_.ErrorGen = NULL; */
    state_ = SOCK_STATE_ALLOCATED;
}

int 
Socket::Connect(char *hostname, Port port)
{
    Address tempAddr;
    
    debug_printf(112, "Sock_Connect %s %d\n", hostname, port);
     
    rHost_ = (RemoteHost *) malloc(sizeof(RemoteHost));
    memset(rHost_, 0, sizeof(RemoteHost));
    if (SetRemoteByName(hostname, port) == SOCK_FAILURE)
	return(SOCK_FAILURE);
   
    if (compatible_ == SOCK_NOT_COMPATIBLE && isCtrl_ != SOCK_CTRL) {
	tcpCtrlChannel_ = new TcpSocket(SOCK_CTRL);
	tcpCtrlChannel_->Listen(SOCK_ANY_PORT);
	udpCtrlChannel_ = new UdpSocket(SOCK_CTRL);
	udpCtrlChannel_->Listen(SOCK_ANY_PORT);
    }

    if (compatible_ == SOCK_NOT_COMPATIBLE) {
	Sock_NameToSin(localName_, 10000, &tempAddr);
	strcpy(localIPAddr_, inet_ntoa(tempAddr.sin_addr));
    }
    side_ = SOCK_SIDE_CLIENT;

    return(SOCK_SUCCESS);
}
     
int 
Socket::Listen(Port port)
{
    int temp;
    int length;

    debug_printf(112, "Sock_Listen %d \n",port);
    lSin_.sin_family = AF_INET;
    lSin_.sin_port = htons(port);
    if (!acceptFlag_)
	lSin_.sin_addr.s_addr = htonl(INADDR_ANY);
    if ((temp = bind_(rfd_,
	      (struct sockaddr *)&lSin_,
	      sizeof(lSin_))) == -1) {
	debug_printf(110, "Sock_listen: error binding socket\n");
	return(SOCK_FAILURE);
    }
    length = sizeof(Address);
    if ((temp = getsockname_(rfd_, (struct sockaddr *)&lSin_,
			    &length)) == -1) {
	debug_printf(110, "Sock_Listen: error getting socket name\n");
	return(SOCK_FAILURE);
    }
    localPort_ = ntohs(lSin_.sin_port);
    debug_printf(109,"listening at inet port %d\n",localPort_);
    state_ = SOCK_STATE_LISTENING;
    return(SOCK_SUCCESS);
}

Sock_Status
Socket::AcceptOW()
{
    Address tempAddr;

    if (compatible_ == SOCK_NOT_COMPATIBLE && isCtrl_ != SOCK_CTRL) {
	tcpCtrlChannel_ = new TcpSocket(SOCK_CTRL);
	tcpCtrlChannel_->Listen(SOCK_ANY_PORT);
	udpCtrlChannel_ = new UdpSocket(SOCK_CTRL);
	udpCtrlChannel_->Listen(SOCK_ANY_PORT);
    }

    if (compatible_ == SOCK_NOT_COMPATIBLE) {
	Sock_NameToSin(localName_, 10000, &tempAddr);
	strcpy(localIPAddr_, inet_ntoa(tempAddr.sin_addr));
    }

    rHost_ = (RemoteHost *) malloc(sizeof(RemoteHost));
    memset(rHost_, 0, sizeof(RemoteHost));

    return(SOCK_SUCCESS);
}    

Socket *
Sock_Accept()
{
    debug_printf(0, "Sock_Accept - wrong func\n");
    return(NULL);
}

int 
Socket::Close()
{
    int temp = SOCK_SUCCESS;
    RemoteHost *rhPtr;
     
    debug_printf(112, "Sock_Close\n");
    if (state_ == SOCK_STATE_CLOSED)
	return(SOCK_SUCCESS);

    /* send "farewell" message to first rhost */
    while (numActiveRHost_ > 0) {
	rhPtr=rHost_;
	if (rhPtr == NULL)
	    debug_printf(101,"Sock_close1: # of rhost error\n");
	if (compatible_ == SOCK_NOT_COMPATIBLE) 
	    CtrlSendFarewell(rhPtr);
	rHost_ = rhPtr->next;
	numActiveRHost_--;
	if (rhPtr->tcpCtrl != NULL) {
	    temp = rhPtr->tcpCtrl->Close();
	    delete rhPtr->tcpCtrl;
	}
	free(rhPtr->remoteName);
	free(rhPtr);
	  
    }
     
    if (rHost_ != NULL)
	debug_printf(101,"Sock_close: # of rhost error\n");
    if (tcpCtrlChannel_ != NULL) {
	tcpCtrlChannel_->Close();
	delete tcpCtrlChannel_;
    }
    if (udpCtrlChannel_ != NULL) {
	udpCtrlChannel_->Close();
	delete udpCtrlChannel_;
    }

    if (sfd_ != -1) {
	temp = shutdown(sfd_, 2);
	close(sfd_);
    }

    if (rfd_ != -1 && rfd_ != sfd_) {
	temp = shutdown(rfd_, 2);
	close(rfd_);
    }

    state_ = SOCK_STATE_CLOSED;
    return((int)temp);

}

Socket::~Socket()
{
    if (state_ != SOCK_STATE_CLOSED)
	Close();
}


